import { Button } from '@material-ui/core'
import euroCrossedIcon from 'assets/icons/ico/euro-crossed.svg'
import euroIcon from 'assets/icons/ico/euro.svg'
import classNames from 'classnames'
import StyledIcon from 'components/CommonKit/Icon/StyledIcon'
import { useClient } from 'cozy-client'
import { useI18n } from 'cozy-ui/transpiled/react/providers/I18n'
import { FluidType, TimeStep } from 'enums'
import { DateTime } from 'luxon'
import React, { useEffect, useState } from 'react'
import ConsumptionService from 'services/consumption.service'
import { useAppSelector } from 'store/hooks'
import './WaterPricing.scss'
import { WaterPricingModal } from './WaterPricingModal'

// In m³
const MAX_FREE = 12
const MAX_REGULAR = 180

export const WaterPricing = () => {
  const { t } = useI18n()
  const client = useClient()
  const { selectedDate } = useAppSelector(state => state.ecolyo.chart)
  const [showModal, setShowModal] = useState(false)
  const [consumption, setConsumption] = useState(0)

  const pricing =
    consumption > 180 ? 'double' : consumption > 12 ? 'regular' : 'free'
  const isFreeExceeded = consumption >= MAX_FREE
  const isRegularExceeded = consumption >= MAX_REGULAR

  const freePercentage = Math.min((consumption / MAX_FREE) * 100, 100)
  // threshold of 30% to display icon
  const freeWithThreshold = Math.max(30, freePercentage)

  const regularPercentage = Math.min((consumption / MAX_REGULAR) * 100, 100)
  const regularWithThreshold = Math.max(10, regularPercentage)

  const year = Number(selectedDate.toFormat('y'))

  useEffect(() => {
    async function fetchData() {
      const cs = new ConsumptionService(client)
      const startDate = DateTime.local(year, 1, 1)
      const endDate = DateTime.local(year, 12, 31)

      const dataLoad = await cs.getGraphData({
        fluidTypes: [FluidType.WATER],
        timeStep: TimeStep.YEAR,
        timePeriod: { startDate, endDate },
      })

      if (!dataLoad?.actualData) return null

      const rounded = Math.ceil(dataLoad.actualData[0].value / 100) / 10
      setConsumption(rounded)
    }
    fetchData()
  }, [client, year])

  return (
    <div className="pricing-root">
      <div className="pricing-container">
        <div className="row">
          <span className="year text-16-bold">
            {t('consumption.water_pricing.year', { year })}
          </span>
          <Button className="btnText" onClick={() => setShowModal(true)}>
            {t('consumption.water_pricing.more')}
          </Button>
        </div>

        <div>
          <div className="gauges">
            <div className="gauge-container free">
              <div className="gauge-border">
                <StyledIcon
                  className="iconFree"
                  icon={euroCrossedIcon}
                  size={12}
                />
                <div
                  className={classNames('gauge-content', {
                    rounded: !isFreeExceeded,
                  })}
                  style={{ right: `${100 - freeWithThreshold}%` }}
                />
              </div>
            </div>

            <div className="separator" />

            <div
              className={classNames('gauge-container regular', {
                'no-color': !isFreeExceeded,
              })}
            >
              <div className="gauge-border">
                <StyledIcon
                  className={`iconRegular ${!isFreeExceeded ? '' : 'filled'}`}
                  icon={euroIcon}
                  size={12}
                />
                <div
                  className={classNames('gauge-content', {
                    rounded: !isRegularExceeded,
                  })}
                  style={{ right: `${100 - regularWithThreshold}%` }}
                />
              </div>
            </div>

            <div className="separator" />

            <div
              className={classNames('gauge-container double', {
                'no-color': !isRegularExceeded,
              })}
            >
              <div className="gauge-border">
                <StyledIcon
                  className={`iconDouble ${!isRegularExceeded ? '' : 'filled'}`}
                  icon={euroIcon}
                  size={12}
                />
                <StyledIcon
                  className={`iconDouble ${!isRegularExceeded ? '' : 'filled'}`}
                  icon={euroIcon}
                  size={12}
                />
                <div
                  className="gauge-content rounded"
                  style={{ right: `${40}%` }}
                />
              </div>
            </div>
          </div>

          <div className="limit-container">
            <span className="limit12">{MAX_FREE}m³</span>
            <span className="limit180">{MAX_REGULAR}m³</span>
          </div>
        </div>

        <div>
          <p className={`pricing ${pricing} text-14`}>
            {t(`consumption.water_pricing.${pricing}`)}
          </p>
          <p
            className="consumption text-14"
            dangerouslySetInnerHTML={{
              __html: t('consumption.water_pricing.consumption', {
                consumption,
              }),
            }}
          />
        </div>
        {showModal && (
          <WaterPricingModal handleCloseClick={() => setShowModal(false)} />
        )}
      </div>
    </div>
  )
}
