import { QueryResult } from 'cozy-client'
import { UsageEventType } from 'enums'
import { DateTime } from 'luxon'
import { AddEventParams, UsageEventEntity } from 'models'
import mockClient from 'tests/__mocks__/client.mock'
import { getError } from 'tests/__mocks__/testUtils'
import {
  allUsageEventsData,
  profileSetEventEntitiesData,
  profileSetUsageEventsData,
  usageEventData,
  usageEventEntityData,
} from 'tests/__mocks__/usageEventsData.mock'
import UsageEventService from './usageEvent.service'

const localSpy = jest.spyOn(DateTime, 'local')

describe('UsageEvent service', () => {
  beforeEach(() => {
    localSpy.mockClear()
  })
  describe('addEvent method', () => {
    it('should add event and return it', async () => {
      const { eventDate, ...addEventParams } = usageEventEntityData
      const params: AddEventParams = {
        ...addEventParams,
        startDate: addEventParams.startDate
          ? DateTime.fromISO(addEventParams.startDate, {
              zone: 'utc',
            })
          : undefined,
      }
      const mockDate = DateTime.fromISO(eventDate, {
        zone: 'utc',
      })
      const mockQueryResult: QueryResult<UsageEventEntity> = {
        data: usageEventEntityData,
        bookmark: '',
        next: false,
        skip: 0,
      }
      localSpy.mockReturnValue(mockDate)
      mockClient.create.mockResolvedValueOnce(mockQueryResult)
      const result = await UsageEventService.addEvent(mockClient, params)
      expect(result).toEqual(usageEventData)
    })
  })
  describe('updateUsageEventsAggregated method', () => {
    it('should return true Connection events', async () => {
      const mockQueryResult: QueryResult<UsageEventEntity> = {
        data: usageEventEntityData,
        bookmark: '',
        next: false,
        skip: 0,
      }
      mockClient.save.mockResolvedValue(mockQueryResult)
      const result = await UsageEventService.updateUsageEventsAggregated(
        mockClient,
        allUsageEventsData
      )
      expect(result).toEqual(true)
    })
    it('should throw an error', async () => {
      mockClient.save.mockRejectedValue(new Error())
      const error = await getError(async () =>
        UsageEventService.updateUsageEventsAggregated(
          mockClient,
          allUsageEventsData
        )
      )
      expect(error).toEqual(new Error('Could not update all events'))
    })
  })
  describe('getEvents method', () => {
    it('should return all ProfileSet events', async () => {
      const mockQueryResult: QueryResult<UsageEventEntity[]> = {
        data: profileSetEventEntitiesData,
        bookmark: '',
        next: false,
        skip: 0,
      }
      mockClient.query.mockResolvedValueOnce(mockQueryResult)
      const result = await UsageEventService.getEvents(mockClient, {
        type: UsageEventType.PROFILE_SET_EVENT,
      })
      expect(result).toEqual(profileSetUsageEventsData)
    })
  })
  describe('addEventIfDoesntExist method', () => {
    it('should not add event', async () => {
      const mockQueryResult: QueryResult<UsageEventEntity[]> = {
        data: profileSetEventEntitiesData,
        bookmark: '',
        next: false,
        skip: 0,
      }
      mockClient.query.mockResolvedValueOnce(mockQueryResult)
      const result = await UsageEventService.addEventIfDoesntExist(
        mockClient,
        { type: UsageEventType.PROFILE_SET_EVENT },
        { id: { $gt: 0 } }
      )
      expect(result).toEqual(null)
    })
    it('should add event', async () => {
      const mockQueryResult: QueryResult<UsageEventEntity[]> = {
        data: [],
        bookmark: '',
        next: false,
        skip: 0,
      }
      mockClient.query.mockResolvedValueOnce(mockQueryResult)
      const mockQueryResult2: QueryResult<UsageEventEntity> = {
        data: usageEventEntityData,
        bookmark: '',
        next: false,
        skip: 0,
      }
      mockClient.create.mockResolvedValueOnce(mockQueryResult2)
      const result = await UsageEventService.addEventIfDoesntExist(
        mockClient,
        { type: UsageEventType.PROFILE_SET_EVENT },
        { id: { $gt: 0 } }
      )
      expect(result).toEqual(usageEventData)
    })
  })
})
